//
//  blc_network.cpp
//  network
//
//  Created by Arnaud Blanchard on 30/05/2015.
//

#include "blc_network.h"
#include "blc_mem.h"
#include "blc_realtime.h"

#include <unistd.h>
#include <pthread.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <string>

using namespace std;

static int blc_vset_iovec(struct iovec *iov, void *data, size_t size, va_list arguments){
    
    int len;
    for(len=0; data != NULL; len++)
    {
        iov[len].iov_base = data;
        iov[len].iov_len = size;
        data = va_arg(arguments, void*);
        size = va_arg(arguments, size_t);
    }
    return len;
    
}
int blc_set_iovec(struct iovec *iovec, void *data, size_t size, ...)
{
    int len;
    va_list arguments;
    
    va_start(arguments, size);
    len = blc_vset_iovec(iovec, data, size, arguments);
    va_end(arguments);
    return len;
}


void blc_set_msghdr(struct msghdr *msghdr, void *data, size_t size, ...)
{
    va_list arguments;
    
    va_start(arguments, size);
    msghdr->msg_iovlen=blc_vset_iovec(msghdr->msg_iov, data, size, arguments);
    va_end(arguments);
}


void blc_network::create_ip4_socket(const char* address_name, const char *port_name, int socket_type){
      struct addrinfo hints, *results;

      int ret;
      CLEAR(hints);
      hints.ai_flags = AI_NUMERICSERV; //We use only port number not service name ( faster )
      hints.ai_family = PF_INET;
      hints.ai_socktype = socket_type;
      
      ret=getaddrinfo(address_name, port_name, &hints, &results);
      if (ret !=0) EXIT_ON_ERROR(gai_strerror(ret));
      if(results->ai_next != NULL) EXIT_ON_ERROR("There is more than one possible address. It is not yet implemented.");
      memcpy(&remote_address, (void*)results->ai_addr, results->ai_addrlen);
      remote_address_length=results->ai_addrlen;
      SYSTEM_ERROR_CHECK(socket_fd = socket(results->ai_family, results->ai_socktype, results->ai_protocol), -1, NULL); //We suppose there is only one result in 'results'
    freeaddrinfo(results);
}



void  blc_network::allocate_system_size_buffer(){
    size_t system_buffer_size=0;
    socklen_t len=sizeof(system_buffer_size);
    SYSTEM_ERROR_CHECK(getsockopt(socket_fd, SOL_SOCKET, SO_RCVBUF, &system_buffer_size, &len), -1, nullptr);
    allocate(system_buffer_size); //No need to free
}

void blc_network::get_remote_address(char *channel_name, socklen_t name_size, char *port_name, socklen_t port_name_size) const{
    int ret;
    
    ret=getnameinfo((const struct sockaddr *)&remote_address, remote_address_length, channel_name, name_size, port_name, port_name_size, NI_NUMERICSERV);
    if (ret !=0) EXIT_ON_ERROR(gai_strerror(ret)); //, 0, "setting address: %s", address_name));
}

void blc_network::recv_buffer_block(void *buffer, size_t buffer_size){
    ssize_t ret;
    SYSTEM_ERROR_CHECK(ret=recvfrom(socket_fd, buffer, buffer_size, MSG_TRUNC, (struct sockaddr*)&remote_address, &remote_address_length), -1, "Socket '%d', data: '%p', size: '%lu'", socket_fd, data, size);
    if ((size_t)ret > buffer_size) EXIT_ON_ERROR("Buffer too small '%lu' bytes with a buffer of '%lu' bytes",  ret, size);
    received_size=ret;
}

void blc_network::recv_block(){
    recv_buffer_block(data, size);
}


bool blc_network::recv_buffer_timeout(void *buffer, size_t buffer_size, uint64_t us_timeout){
    fd_set fdset={{0}};
    ssize_t ret;
    struct timeval timeout;
    div_t result;
    
    FD_SET(socket_fd, &fdset);
    result=div(us_timeout, 1000000);
    timeout.tv_sec=result.quot;
    timeout.tv_usec=result.rem;
    SYSTEM_ERROR_CHECK(ret=select(socket_fd+1, &fdset, NULL, NULL, &timeout), -1, "Waiting server channel data"); //It would be nice to check errorfds
    if (ret==0) return false;
    else{
        SYSTEM_ERROR_CHECK(ret=recvfrom(socket_fd, buffer, buffer_size, MSG_TRUNC, (struct sockaddr*)&remote_address, &remote_address_length), -1, "Socket '%d', data: '%p', size: '%lu'", socket_fd, data, size);
        if ((size_t)ret > size) EXIT_ON_ERROR("Buffer too small '%lu' bytes with a buffer of '%lu' bytes",  ret, size);
     }
    return true;
}

bool blc_network::recv_timeout(uint64_t us_timeout){
    return recv_buffer_timeout(data, size, us_timeout );
}

void blc_network::send_buffer(void const* buffer, size_t buffer_size){
    ssize_t ret;
    size_t system_buffer_size=0;
    socklen_t len=sizeof(system_buffer_size);
    
    ret=sendto(socket_fd, buffer, buffer_size, NO_FLAG, (struct sockaddr*)&remote_address, remote_address_length);
    if (ret==-1){
        SYSTEM_ERROR_CHECK(getsockopt(socket_fd, SOL_SOCKET, SO_SNDBUF, &system_buffer_size, &len), -1, nullptr);
        if (errno==EMSGSIZE) EXIT_ON_SYSTEM_ERROR("socket_fd '%d', request size '%u' SNDBUFFER '%d'", socket_fd, buffer_size, system_buffer_size );
    }
    else if ((size_t)ret!=buffer_size) EXIT_ON_ERROR("Only '%l' bytes sent instead of '%lu'", ret, size);
}

void blc_network::send(ssize_t send_size){
    if (this->data==NULL) EXIT_ON_ERROR("You have not allocated buffer.");
    if (send_size==-1) send_size=size; //By default we send all
    if ((size_t)send_size > this->size) EXIT_ON_ERROR("You try to send more '%ld' than the buffer size '%lu'", send_size, size);
    send_buffer(this->data, send_size);
}

blc_network::~blc_network(){
    close(socket_fd);
}

